package com.ibm.sample.hpel;

import java.util.ArrayList;
import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.logging.Level;

import com.ibm.websphere.logging.hpel.reader.LogQueryBean;
import com.ibm.websphere.logging.hpel.reader.LogRepositoryException;
import com.ibm.websphere.logging.hpel.reader.LogRepositoryRuntimeException;
import com.ibm.websphere.logging.hpel.reader.MergedRepository;
import com.ibm.websphere.logging.hpel.reader.RepositoryLogRecord;
import com.ibm.websphere.logging.hpel.reader.RepositoryReader;
import com.ibm.websphere.logging.hpel.reader.RepositoryReaderImpl;
import com.ibm.websphere.logging.hpel.reader.ServerInstanceLogRecordList;

/**
 * simple sample of using the Read API.  To make this run requires only the com.ibm.hpel.logging.jar in the buildpath
 * and runtime classpath. In practice, more information will be printed for each record and special formatting will be done,
 * this is primarily a sample to enable someone to begin using the technology with a simple self-contained program.  It goes
 * through all ServerInstances in a repository.  It does not handle a z/OS condition of child processes.  See other samples
 * for that.
 */
public class ReaderSamplesForExercises {
	private static String baseRepository = null ;
	private static String mergeRepository = null ;

	public static void main(String[] args) {
		if (args.length < 2)  usage() ;
		baseRepository = args[0] ;
		int whichTest = Integer.parseInt(args[1]) ;
		if (whichTest == 3)
			if (args.length < 3)  usage() ;
			else  mergeRepository = args[2] ; 
		switch (whichTest) {
		case 1: exercise1() ; break ;
		case 2: exercise2() ; break ;
		case 3: exercise3() ; break ;
		case 4: exercise4() ; break ;
		case 5: exercise5() ; break ;
		default: System.out.println("Invalid test of: "+args[1]+" specified");
		}
		
	}
	
	private static void usage() {
		System.out.println("Usage:  java ReaderSamplesForExercises <logRepositoryDirectory> <test number> <addedArgs (test specific)>");
		System.out.println("ie:  java ReaderSamplesForExercises /opt/IBM/WebSphere/AppServer/profiles/default/logs/server1 1");
		System.out.println("TestOptions match those in exercises at: https://wst.austin.ibm.com/wstdev/4176.html");
		System.out.println("option 3 requires an added arg for the second repository directory");
		System.exit(-99) ;

	}
	
	/**
	 * retrieve all WARNING and SEVERE messages in the repository and print some data. Note the pattern as it is
	 * common to most examples.  Note one key difference in getLogLists vs getLogListForServerInstance.
	 */
	private static void exercise1() {
		// Create a repository reader (requires base directory of repository 
		RepositoryReader logRepository = new RepositoryReaderImpl(baseRepository) ;		
		// Get iterator of server instances (start/stop of the server) extracting all log messages with
		// severity between INFO and SEVERE.  Lots of different filtering options, this is just one sample
		Iterable<ServerInstanceLogRecordList> repResults = null;
		try {
			repResults = logRepository.getLogLists(Level.WARNING, Level.SEVERE);
		} catch (LogRepositoryException e) {
			System.out.println("Exception in get log lists: "+e);
		}
				// Go through each server instance
		for (ServerInstanceLogRecordList pidRecords: repResults) {		// For each list (server lifeCycle)
				// For each server instance, go through the records
			try {
				for (RepositoryLogRecord repositoryLogRecord : pidRecords) {
						// Just printing some key information here.  Note that the repositoryRecord exposes all fields
						// with simple get methods
						System.out.println("  "+new Date(repositoryLogRecord.getMillis())+"  "+
							repositoryLogRecord.getThreadID()+"  "+repositoryLogRecord.getFormattedMessage());
				}
			} catch (LogRepositoryRuntimeException lrre) {
				System.out.println("Exception while retrieving data: "+lrre);
			}
		}
	}

	/**
	 * retrieve all WARNING and SEVERE messages in the latest server instance in the repository.  Note that
	 * getLogListForServerInstance with a null Date parm is equivalent to getLogListForCurrentServerInstance, but
	 * it allows for filtering
	 */
	private static void exercise2() {
		// Create a repository reader (requires base directory of repository 
		RepositoryReader logRepository = new RepositoryReaderImpl(baseRepository) ;	
		// Get iterator of server instances (start/stop of the server) extracting all log messages with
		// severity between INFO and SEVERE.  Lots of different filtering options, this is just one sample
		ServerInstanceLogRecordList silrl = null;
		try {
			silrl = logRepository.getLogListForServerInstance((Date)null, Level.WARNING, Level.SEVERE) ;
		} catch (LogRepositoryException e) {
			System.out.println("Exception in get log lists: "+e);
		}
				// Only one server instance, so can use it directly
		try {
			for (RepositoryLogRecord repositoryLogRecord : silrl) {
					// Just printing some key information here.  Note that the repositoryRecord exposes all fields
					// with simple get methods
					System.out.println("  "+new Date(repositoryLogRecord.getMillis())+"  "+
						repositoryLogRecord.getThreadID()+"  "+repositoryLogRecord.getFormattedMessage());
			}
		} catch (LogRepositoryRuntimeException lrre) {
			System.out.println("Exception while retrieving data: "+lrre);
		}
	}

	/**
	 * retrieve and merge. Could use similar getLogListForServerInstance as done in earlier examples, but I realized that
	 * that would NOT highlight the LogQueryBean ... so I am using that here.  Most serious filtering is done with that
	 * and showing the same shortcut in all examples is shortChanging the reader and somewhat boring as well.
	 */
	private static void exercise3() {
		// Put them in array which can handle N repository bases and avoid me writing code twice (not to suggest I'm lazy)
		String [] repositoryLocation = { baseRepository, mergeRepository } ;
		
		// Each query will result in a ServerInstanceLogRecordList
		ServerInstanceLogRecordList [] silrlArray = new ServerInstanceLogRecordList[repositoryLocation.length]  ;
		
		for (int i = 0; i < repositoryLocation.length; i++) {
			RepositoryReader logRepository = new RepositoryReaderImpl(repositoryLocation[i]) ;	
			// Get iterator of server instances (start/stop of the server) extracting all log messages with
			// severity between INFO and SEVERE.  Lots of different filtering options, this is just one sample
			try {
				LogQueryBean logQueryBean = new LogQueryBean() ;
				logQueryBean.setLevels(Level.INFO, Level.SEVERE) ;
				silrlArray[i] = logRepository.getLogListForServerInstance((Date)null, Level.WARNING, Level.SEVERE) ;
			} catch (LogRepositoryException e) {
				System.out.println("Exception in get log lists: "+e);
			}
		}
		
		MergedRepository result = new MergedRepository(silrlArray) ;  	// Merge the current serverInstance from each server
		System.out.println("ServerNm/Pid    Date/Time    Thread    Msg");
		try {
			for (RepositoryLogRecord repositoryLogRecord : result) {		// For each record in new merged set
					// Note that each record references back to the header for that file to access key info there
				String serverNm = result.getHeader(repositoryLogRecord).getProperty(ServerInstanceLogRecordList.HEADER_SERVER_NAME) ;
				String pid = result.getHeader(repositoryLogRecord).getProperty(ServerInstanceLogRecordList.HEADER_PROCESSID) ;
					// Just printing some key information here.  Note that the repositoryRecord exposes all fields
					// with simple get methods and gives access to the header information associated with each as well
				System.out.println(serverNm+"/"+pid+"  "+new Date(repositoryLogRecord.getMillis())+"  "+
					repositoryLogRecord.getThreadID()+"  "+repositoryLogRecord.getFormattedMessage());
			}
		} catch (LogRepositoryRuntimeException lrre) {
			System.out.println("Exception while retrieving data: "+lrre);
		}
	}
	
	/**
	 * retrieve log for the controller and list out all servants that you could pull logs for (and will in exercise5)
	 */
	private static void exercise4() {
		// logQueryBean has many filtering options which can be combined for complex queries
		LogQueryBean logQueryBean = new LogQueryBean() ;
		logQueryBean.setLevels(Level.WARNING, Level.SEVERE) ;
		
		RepositoryReader logRepository = new RepositoryReaderImpl(baseRepository) ;		
				// Pull from just the current instance (null date) of the server for merging on appropriate times
				// Could pull from all or use filter criteria that would select several, this is for simplicity
		try {
					// Read records from the current server instance in repository that match query bean
			ServerInstanceLogRecordList silrl = logRepository.getLogListForServerInstance((Date)null, logQueryBean) ;
					// Add resultSet to ArrayList
					// Look for child resultSets (ie: z/OS servants and adjuncts)
	    	Map<String, ServerInstanceLogRecordList>servantMap = silrl.getChildren() ;
	    			// Each child has its own resultSet, add each resultSet to the arrayList (no affect on distributed)
	        Iterator<String> servantKeys = servantMap.keySet().iterator() ;
	        while (servantKeys.hasNext()) {
	        	String label = servantKeys.next() ;			// Map label has key information to identify which child process
	        	System.out.println("Retrieved servant label: "+label);
	        }
		} catch (LogRepositoryException e) {
			System.out.println("Exception on getLogListForServerInstance: "+e);
			System.exit(-99) ;
		}
	}
	
	/**
	 * retrieve and merge for a single serverInstance of a z/OS repository.  Note that z/OS has multiple processes per
	 * server, so you can merge w/out using multiple root repositories.  Note also, however, that you cannot predict the
	 * number of server processes to merge since any start/stop of a servant creates a child serverInstance.
	 */
	private static void exercise5() {
		ArrayList <ServerInstanceLogRecordList> silrlList = new ArrayList<ServerInstanceLogRecordList>() ;
		// logQueryBean has many filtering options which can be combined for complex queries
		LogQueryBean logQueryBean = new LogQueryBean() ;
		logQueryBean.setLevels(Level.WARNING, Level.SEVERE) ;
		
		RepositoryReader logRepository = new RepositoryReaderImpl(baseRepository) ;		
				// Pull from just the current instance (null date) of the server for merging on appropriate times
				// Could pull from all or use filter criteria that would select several, this is for simplicity
		try {
					// Read records from the current server instance in repository that match query bean
			ServerInstanceLogRecordList silrl = logRepository.getLogListForServerInstance((Date)null, logQueryBean) ;
					// Add resultSet to ArrayList
			silrlList.add(silrl) ;
					// Look for child resultSets (ie: z/OS servants and adjuncts)
	    	Map<String, ServerInstanceLogRecordList>servantMap = silrl.getChildren() ;
	    			// Each child has its own resultSet, add each resultSet to the arrayList (no affect on distributed)
	        Iterator<String> servantKeys = servantMap.keySet().iterator() ;
	        while (servantKeys.hasNext()) {
	        	String label = servantKeys.next() ;			// Map label has key information to identify which child process
	        	ServerInstanceLogRecordList servantRecs = servantMap.get(label) ;		// Pull the child process serverInstance
	        	silrlList.add(servantRecs) ;
	        }
		} catch (LogRepositoryException e) {
			System.out.println("Exception on getLogListForServerInstance: "+e);
			System.exit(-99) ;
		}
					// Convert the arrayList into an array
		ServerInstanceLogRecordList [] silrlArray = new ServerInstanceLogRecordList[silrlList.size()] ;
		silrlArray = silrlList.toArray(silrlArray) ;
					// Merge records from all resultSets contained in the array
		MergedRepository result = new MergedRepository(silrlArray) ;  	// Merge the current serverInstance from each server
		System.out.println("ServerNm/Pid    Date/Time    Thread    Msg");
		try {
			for (RepositoryLogRecord repositoryLogRecord : result) {		// For each record in new merged set
					// Note that each record references back to the header for that file to access key info there
				String serverNm = result.getHeader(repositoryLogRecord).getProperty(ServerInstanceLogRecordList.HEADER_SERVER_NAME) ;
				String pid = result.getHeader(repositoryLogRecord).getProperty(ServerInstanceLogRecordList.HEADER_PROCESSID) ;
					// Just printing some key information here.  Note that the repositoryRecord exposes all fields
					// with simple get methods and gives access to the header information associated with each as well
				System.out.println(serverNm+"/"+pid+"  "+new Date(repositoryLogRecord.getMillis())+"  "+
					repositoryLogRecord.getThreadID()+"  "+repositoryLogRecord.getFormattedMessage());
			}
		} catch (LogRepositoryRuntimeException lrre) {
			System.out.println("Exception while retrieving data: "+lrre);
		}
	}
}